use std::{collections::HashMap, sync::Arc};

use axum::async_trait;
use como_core::projects::ProjectService;
use como_domain::{
    projects::{mutation::CreateProjectMutation, queries::GetProjectQuery, ProjectDto},
    users::User,
};
use tokio::sync::Mutex;

pub struct DefaultProjectService {}

impl DefaultProjectService {
    pub fn new() -> Self {
        Self {}
    }
}

#[async_trait]
impl ProjectService for DefaultProjectService {
    async fn get_project(&self, _query: GetProjectQuery) -> anyhow::Result<ProjectDto> {
        todo!()
    }
    async fn get_projects(&self, user: &User) -> anyhow::Result<Vec<ProjectDto>> {
        todo!()
    }
    async fn create_project(
        &self,
        name: CreateProjectMutation,
        user: &User,
    ) -> anyhow::Result<ProjectDto> {
        todo!()
    }
}

pub struct MemoryProjectService {
    project_store: Arc<Mutex<HashMap<String, ProjectDto>>>,
}

impl MemoryProjectService {
    pub fn new() -> Self {
        Self {
            project_store: Arc::new(Mutex::new(HashMap::new())),
        }
    }
}

#[async_trait]
impl ProjectService for MemoryProjectService {
    async fn get_project(&self, query: GetProjectQuery) -> anyhow::Result<ProjectDto> {
        let ps = self.project_store.lock().await;
        Ok(ps
            .get(&query.project_id.to_string())
            .ok_or(anyhow::anyhow!("could not find project"))?
            .clone())
    }
    async fn get_projects(&self, user: &User) -> anyhow::Result<Vec<ProjectDto>> {
        Ok(self
            .project_store
            .lock()
            .await
            .values()
            .filter(|p| p.user_id == user.id)
            .cloned()
            .collect::<_>())
    }

    async fn create_project(
        &self,
        mutation: CreateProjectMutation,
        user: &User,
    ) -> anyhow::Result<ProjectDto> {
        let mut ps = self.project_store.lock().await;
        let project = ProjectDto {
            id: uuid::Uuid::new_v4(),
            name: mutation.name,
            description: None,
            user_id: user.id.clone(),
        };

        ps.insert(project.id.to_string(), project.clone());
        Ok(project)
    }
}
